% Combines the functionality of multipulse_frog_from_file.m and
% make_pulse_train.m into a single script that generates a pulse train and
% a FROG measurement of the pulse train.  Does not generate any of the 
% pulse train statistics found in make_pulse_train.m .
%
% Authors: Justin Ratner and Michelle Rhodes
% Georgia Institute of Technology
% email: mrhodes3@gatech.edu
% Last revision: 5/13/2013

clear all

n           = 4096;     % size of the array
num_pulses  = 5000;     % number of pulses to average over

%choose SHG, X, or PG
method = 'SHG';               % nonlinearity
samp = 4;
crop = 4;

%Constant by which the unstable field is multiplied.
energy_coeff = 14;     

%Width of Gaussian time window.  This parameter controls how long the
% random component is in time.  The average RMS width of the random 
% components will be approximately equal to this.
time_width = 100;   

%
% Create deterministic Gaussian component of pulse, want 20 dt FWHM
%
pw = sqrt(n/pi/4.5);
x = -n/2:n/2-1;
etg = exp(-x.^2 / pw^2);
ewg = 1/sqrt(n)*fftc(etg);
spectrumg = abs(ewg).^2;

% Envelopes for the random pulses
time_envelope = exp(-x.^2/time_width^2);
freq_envelope = sqrt(spectrumg);

nc = n/crop;
indices = (n/2-nc/2):samp:(n/2+nc/2-1);
n_trace = length(indices);

if strcmp(method(1),'X')
    nl = 'PG';
else
    nl = method;
end
g = gate(nl);
if strcmp(method(1), 'X')
    g = @gate_x;
end

et = zeros(num_pulses, n);
ewr = freq_envelope.*exp(1i*rand(1,n)*2*pi); %same spectrum, random phase
etr = sqrt(n)*ifftc(ewr);
etr = energy_coeff*etr.*(time_envelope); %apply a the time envelope and adjust energy
et(1, :) = etg + etr;
isig0 = calc_isig_wT(et(1,indices), g(et(1,indices)));

isig = isig0;
if num_pulses > 1
    for k = 2:num_pulses
        
        ewr = freq_envelope.*exp(1i*rand(1,n)*2*pi); %same spectrum, random phase
        etr = sqrt(n)*ifftc(ewr);
        etr = energy_coeff*etr.*(time_envelope); %apply a the time envelope and adjust energy
        et(k, :) = etg + etr;
        
        isig1 = calc_isig_wT(et(k,indices), ...
            g(et(k,indices)));
        
        isig = (isig + isig1);
        
        if k == 2 || mod(k, 1000) == 0
            fprintf('Integrating trace %d of %d... \n', k, num_pulses)
        end
    end
end
isig = quickscale(isig);

options = struct('Display',		'print', ...
	'DisplayStep',	25, ...
	'DisplayLimit',	100, ...
	'Domain',		'time', ...
	'CorrectG',		'on', ...
	'KeepBest',		'g', ...
	'MaxIter',		1000, ...
	'MinimumG',		2e-4, ...
	'MinimumZ',		0.0 );
x = -n/2:n/2-1;
xsamp = samp*(-n_trace/2:n_trace/2-1);
et_guess = rand(1,n_trace) + 1i*rand(1,n_trace);
switch method
    case 'SHG'
        [etb_retr, et_retr] = QuickFROG_tT(sqrt(isig), et_guess, ...
            xsamp, xsamp, options);
    case 'PG'
        [etb_retr, et_retr] = QuickFROG_PG_tT(sqrt(isig), et_guess, ...
            xsamp, xsamp, options);
    case 'X'
        Gate = g(et_guess);
        [etb_retr, et_retr] = QuickFROG_tTX(sqrt(isig), et_guess, ...
            xsamp, xsamp, xsamp, Gate, options);
end
isig_retr = calc_isig_wT(et_retr, g(et_retr));

[mu fit_error] = fminsearch(@(params) Gprime_min(params, isig, isig_retr), 1);

Gprime_err = sqrt( trapz(trapz(abs(isig - mu*isig_retr).^2))/trapz(trapz(isig.^2)) );

tlim = 150;
wlim = 150;

intensity = quickscale(abs(et_retr).^2);
width2 = fwhm(intensity)*samp;

fprintf('Real FWHM: %d\n', width2);
fprintf('G prime error: %d\n', Gprime_err);

figure;
subplot 221
h = imagesc(xsamp,xsamp,isig);
style_plot(gca, h, 'frog_trace')
set(gca, 'XLim', [-1 1]*tlim, 'YLim', [-1 1]*wlim)
subplot 222
h = imagesc(xsamp,xsamp,isig_retr);
style_plot(gca, h, 'frog_trace')
set(gca, 'XLim', [-1 1]*tlim, 'YLim', [-1 1]*wlim)
subplot 223
et0 = et_retr;
[ax, h1, h2] = plotyy(xsamp, quickscale(abs(et0).^2), ...
                      xsamp, unwrap(angle(et0)));
style_plot(ax, {h1, h2}, 'frog_time')
set(ax, 'XLim', [-1 1]*tlim)
subplot 224
ew0 = fftc(et0);
[ax, h1, h2] = plotyy(xsamp, quickscale(abs(ew0).^2), ...
                      xsamp, unwrap(angle(ew0)));
style_plot(ax, {h1, h2}, 'frog_freq')
set(ax, 'XLim', [-1 1]*wlim)
set(gcf, 'PaperOrientation', 'landscape')
set(gcf, 'PaperSize', [20 20])